import pandas as pd
import matplotlib.pyplot as pl
from matplotlib.ticker import FuncFormatter


class AnalyzeResults(object):

    def __init__(self, filename):
        
        self.df = pd.read_csv(filename, parse_dates=[1], infer_datetime_format=True)
        

    def get_annual_sales(self):
        return self.df.cost.groupby(self.df.date_time.dt.year).sum()


    def get_hourly_customers(self):
        return self.df.sale_id.groupby(self.df.date_time.dt.hour).count()


    def get_postage_method_counts(self):
        return self.df.quantity.groupby(self.df['postage method']).sum()


    def get_top_destinations(self, n=5): 
        return self.df.quantity.groupby(self.df.destination).sum().nlargest(n)


    def display_results(self, make_plots=False):
        annual_sales = self.get_annual_sales()
        hourly_customers = self.get_hourly_customers()
        postage_counts = self.get_postage_method_counts()
        top_destinations = self.get_top_destinations()
        
        print("")
        print("----------Annual Gross Sales----------")
        for year, amount in annual_sales.iteritems():
            print("  {}: {:.2f}".format(year, amount))
        
        print("")
        print("-------------Hourly Sales-------------")
        for hour, count in hourly_customers.iteritems():
            print("  {:02d}: {}".format(hour, count))
        
        print("")
        print("------Postage Methods Popolarity------")
        for method, count in postage_counts.iteritems():
            print("  {:<22}: {}".format(method, count))
        
        print("")
        print("-----Top 5 Destination Countries------")
        for i, country in enumerate(top_destinations.index, start=1):
            print("  {}. {}".format(i, country))
            
        if make_plots:
            fig = pl.figure(figsize=(12,8), frameon=False)
            ax1 = fig.add_subplot(221)
            ax2 = fig.add_subplot(222)
            ax3 = fig.add_subplot(223)
            ax4 = fig.add_subplot(224)
            
            ax1.set_xlabel('Year')
            ax1.set_ylabel('Gross Sales')
            ax1.set_xticks(annual_sales.index)
            ax1.set_xticklabels(annual_sales.index.map(str))
            ax1.get_yaxis().set_major_formatter(FuncFormatter(lambda x, p: '{:.0f}k'.format(x*1e-3)))
            ax1.plot(annual_sales.index, annual_sales.values, '-o')
            
            ax2.set_xlim(0, 24)
            ax2.set_xlabel('Hour')
            ax2.set_ylabel('Total Customers')
            ax2.bar(hourly_customers.index, hourly_customers.values)
            
            ax3.pie(postage_counts.values, labels=postage_counts.index.str.replace(' ', '\n', 1), 
                    autopct='%1.1f%%', startangle=150)
            ax3.set_aspect(1)
            
            index = top_destinations.reset_index().index[::-1]
            colors = ['red', 'blue', 'orange', 'green', 'purple']
            ax4.set_yticklabels(top_destinations.index.str.replace(' ', '\n', 1))
            ax4.set_yticks(index)
            ax4.set_xlabel('Stamps Purchased')
            ax4.barh(index, top_destinations.values, align='center', color=colors)
            pl.show()
        
        
if __name__ == '__main__':
    
    analyzer = AnalyzeResults('sales_history.csv')
    analyzer.display_results(True)

    
    
