import os
import re
import csv
import bisect
import collections


PRICE_DATA_DIR = 'price_data'
COUNTRY_ZONES_FILE = 'Countries and Zones.csv'


def read_csv(fname):
    ''' reads a csv file with a header into a list of dicts
    '''
    with open(fname, 'r') as fp:
        reader = csv.DictReader(fp)
        data = list(reader)
    return data


def read_country_zones(fname):
    return {row['Destination country'].lower(): row['Zones'] for row in read_csv(fname)}
    
    
class PriceGuide(object):

    country_lookup = read_country_zones(COUNTRY_ZONES_FILE)

    def __init__(self, filename, preprocess=False):
        data = read_csv(filename)
        
        if preprocess:
            data = self.preprocess(data)
        self.lookup = self.parse_table(data)
        self.weight_breakpoints = list(self.lookup)
        self.max_weight = self.weight_breakpoints[-1]
            
    def preprocess(self, table):
        processed = []
        for row in table:
            payload = {}
            for key in row:
                if key == 'Weight':
                    payload[key] = 'up to ' + row[key].lower().split('up to', 1)[1]
                else:
                    zones = key.split('Zone')[1]
                    for zone_number in zones.replace('and', ' ').replace(',', ' ').split():
                        payload['Zone ' + zone_number] = row[key]
                processed.append(payload)
                
        return processed
            
            
    def parse_table(self, table):
        ''' parses a raw data table (list of dicts)
            into a dict of dicts with weights and zones being values
        '''
        mapping = collections.OrderedDict()
        for row in table:
            weight_value = row['Weight'].lower().split('up to', 1)[1]
            key = self.parse_weight(weight_value)
            values = {k: row[k] for k in row if k.startswith('Zone')}
            mapping[key] = values
        return mapping

    @staticmethod
    def parse_weight(value):
        ''' converts weight string like '50g or '1.5 kg' to the amount in kg
        '''
        match = re.match(r'\s*([0-9]*\.?[0-9]+)\s*(k?)g\s*', value)
        prefix = {'': 0.001, 'k': 1} 
        if match:
            return prefix[match.group(2)] * float(match.group(1))
        else:
            raise ValueError('Cannot parse weight: {}'.format(value))
    

    def price_lookup(self, weight, country):
        
        if isinstance(weight, str):
            weight = self.parse_weight(weight)
        
        if weight > self.max_weight:
            raise ValueError("No postage above {}kg".format(self.max_weight))
            
        weight_idx = bisect.bisect_left(self.weight_breakpoints, weight)
        base_weight = self.weight_breakpoints[weight_idx]
            
        try:
            zone = self.country_lookup[country.lower()]
            return float(self.lookup[base_weight][zone])
        except KeyError:
            raise ValueError("No postage to " + country)    
            

def load_price_guides():
    
    price_guide_files = {
        'Economy Air Letter'   : 'Economy Letters Price Guide ($).csv',
        'Economy Parcel_by Air': 'Economy Parcel Price Guide_by Air ($).csv',
        'Economy Parcel_by Sea': 'Economy Parcel Price Guide_by Sea ($).csv',
        'Express Letter'       : 'Express Letter Price Guide ($).csv',
        'Express Parcel'       : 'Express Parcel Price Guide ($).csv',
        'Standard Parcel'      : 'Standard Parcel Price Guide ($).csv'
    }
    
    price_guide_types = lambda x: x.split()[1].lower()

    price_guides = {name: PriceGuide(os.path.join(PRICE_DATA_DIR, filename), 
                                     preprocess=(name == 'Economy Air Letter'))
                    for name, filename in price_guide_files.items()
                   } 
    return price_guides


if __name__ == '__main__':
    price_guide = load_price_guides()
    
    
    
